/**
 *  Licensed to the Apache Software Foundation (ASF) under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
/*
 * Auto-generated by J2G Migration. 
 */
package org.apache.geronimo.j2g.sources.dependence.compatibility;

import java.io.IOException;
import java.lang.reflect.Constructor;
import java.security.Principal;
import java.security.acl.Group;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import javax.security.auth.Subject;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.NameCallback;
import javax.security.auth.callback.PasswordCallback;
import javax.security.auth.callback.UnsupportedCallbackException;
import javax.security.auth.login.FailedLoginException;
import javax.security.auth.login.LoginException;
import javax.security.auth.spi.LoginModule;

import org.apache.geronimo.security.realm.providers.GeronimoGroupPrincipal;
import org.apache.geronimo.security.realm.providers.GeronimoUserPrincipal;

public abstract class GeronimoLoginModule implements LoginModule {

	protected Subject subject;

	protected CallbackHandler callbackHandler;

	protected Map sharedState;

	protected Map options;

	protected boolean useFirstPass;

	protected boolean loginOk;

	protected String principalClassName;

	protected Principal unauthenticatedIdentity;

	private Principal identity;

	private char passport[];

	private String algorithm;

	private String charset;

	private String encoding;

	private boolean ignorePasswordCase;

	protected final static int USERNAME_POS = 0;

	protected final static int PASSWORD_POS = 1;

	public GeronimoLoginModule() {
		algorithm = null;
		charset = null;
		encoding = null;
	}

	public boolean abort() throws LoginException {
		return true;
	}

	public boolean commit() throws LoginException {
		if (!loginOk) {
			return false;
		}
		Set heads = subject.getPrincipals();
		Principal identity = getIdentity();
		heads.add(identity);
		Group roleSets[] = getRoleSets();
		for (int i = 0; i < roleSets.length; i++) {
			Group group = roleSets[i];
			String name = group.getName();
			Group subjectGroup = createGroup(name, heads);
			Principal role;
			Enumeration members = group.members();
			while (members.hasMoreElements()) {
				role = (Principal) members.nextElement();
				subjectGroup.addMember(role);
				if(name.equals("Roles")){
					heads.add(new GeronimoGroupPrincipal(role.getName()));
				}
			}
		}
		return true;
	}

	protected Principal getIdentity() {
		return identity;
	}

	public void initialize(Subject subject, CallbackHandler callbackHandler,
			Map sharedState, Map options) {
		this.subject = subject;
		this.callbackHandler = callbackHandler;
		this.sharedState = sharedState;
		this.options = options;
		String passwordStacking = (String) options.get("password-stacking");
		if (passwordStacking != null
				&& passwordStacking.equalsIgnoreCase("useFirstPass")) {
			useFirstPass = true;
		}
		principalClassName = (String) options.get("principalClass");
		String name = (String) options.get("unauthenticatedIdentity");
		if (name != null) {
			try {
				unauthenticatedIdentity = createIdentity(name);
			} catch (Exception e) {
			}
		}
		algorithm = (String) options.get("hashAlgorithm");
		if (algorithm != null) {
			encoding = (String) options.get("hashEncoding");
			if (encoding == null) {
				encoding = "BASE64";
			}
			charset = (String) options.get("hashCharset");
		}
		String flag = (String) options.get("ignorePasswordCase");
		ignorePasswordCase = Boolean.valueOf(flag).booleanValue();
	}

	public boolean login() throws LoginException {
		loginOk = false;
		if (useFirstPass) {
			Object identity = sharedState.get("javax.security.auth.login.name");
			Object credential = sharedState
					.get("javax.security.auth.login.password");
			if (identity != null && credential != null) {
				loginOk = true;
			}
		}
		if (loginOk) {
			Object username = sharedState.get("javax.security.auth.login.name");
			if (username instanceof Principal) {
				identity = (Principal) username;
			} else {
				try {
					identity = createIdentity(username.toString());
				} catch (Exception e) {
					throw new LoginException("Failed to create principal: "
							+ e.getMessage());
				}
			}
			Object password = sharedState
					.get("javax.security.auth.login.password");

			if (password != null) {
				if (password instanceof char[]) {
					passport = (char[]) password;
				} else {
					passport = password.toString().toCharArray();
				}
			}
			return true;
		}
		loginOk = false;

		String username = null;
		String password = null;

		String[] usernameAndPassword = getUsernameAndPassword();

		username = usernameAndPassword[USERNAME_POS];
		password = usernameAndPassword[PASSWORD_POS];
		if (username == null && password == null) {
			identity = unauthenticatedIdentity;
		}
		if (identity == null) {
			try {
				identity = createIdentity(username);
			} catch (Exception e) {
				throw new LoginException("Failed to create principal: "
						+ e.getMessage());
			}
			if (algorithm != null) {
				password = createPasswordHash(username, password);
			}
			String expectedPassword = getUsersPassword();
			if (!validatePassword(password, expectedPassword)) {
				throw new FailedLoginException(
						"Password Incorrect/Password Required");
			}
		}
		if (getUseFirstPass()) {
			sharedState.put("javax.security.auth.login.name", username);
			sharedState.put("javax.security.auth.login.password", passport);
		}
		loginOk = true;
		return loginOk;
	}

	protected String[] getUsernameAndPassword() throws LoginException {
		String usernamePassword[] = { null, null };
		if (callbackHandler == null) {
			throw new LoginException(
					"Error occurs: there are no CallbackHandler available to get information about authentication");
		}
		String username = null;
		String password = null;
		try {
			Callback callbacks[] = { new NameCallback("Username: "), new PasswordCallback("Password: ", false) };
			callbackHandler.handle(callbacks);
			NameCallback nameCallback = (NameCallback) callbacks[0];
			PasswordCallback passwordCallback = (PasswordCallback) callbacks[1];
			username = nameCallback.getName();
			char helperPassword[] = passwordCallback.getPassword();
			if (helperPassword != null) {
				passport = new char[helperPassword.length];
				System.arraycopy(helperPassword, 0, passport, 0,
						helperPassword.length);
				passwordCallback.clearPassword();
				password = new String(passport);
			}
		} catch (IOException ioe) {
			throw new LoginException(ioe.toString());
		} catch (UnsupportedCallbackException uce) {
			throw new LoginException("CallbackHandler does not support: "
					+ uce.getCallback());
		}
		usernamePassword[USERNAME_POS] = username;
		usernamePassword[PASSWORD_POS] = password;
		return usernamePassword;
	}

	protected boolean validatePassword(String inputPassword,
			String expectedPassword) {
		if (inputPassword == null || expectedPassword == null) {
			return false;
		}
		return ignorePasswordCase ? inputPassword
				.equalsIgnoreCase(expectedPassword) : inputPassword
				.equals(expectedPassword);
	}

	protected String createPasswordHash(String username, String password) {
		DigestCallback callback = null;
		String callbackClassName = (String) options.get("digestCallback");
		if (callbackClassName != null) {
			try {
				Class callbackClass = loadSomeClass(callbackClassName);
				callback = (DigestCallback) callbackClass.newInstance();
			} catch (Exception e) {
				SecurityException securityExeption = new SecurityException(
						"Cannot load DigestCallback");
				securityExeption.initCause(e);
				throw securityExeption;
			}
			HashMap optionsMap = new HashMap(options);
			optionsMap.put("javax.security.auth.login.name", username);
			optionsMap.put("javax.security.auth.login.password", password);
			callback.init(optionsMap);
		}

		return PasswordHasher.calculateHash(algorithm, encoding, charset,
				username, password, callback);
	}

	protected boolean getUseFirstPass() {
		return useFirstPass;
	}

	protected Principal getUnauthenticatedIdentity() {
		return unauthenticatedIdentity;
	}

	protected Object getCredentials() {
		return passport;
	}

	public boolean logout() throws LoginException {
		Principal identity = getIdentity();
		Set principals = subject.getPrincipals();
		principals.remove(identity);
		return true;
	}

	protected Principal createIdentity(String username) throws Exception {
		Principal principal = null;
		if (principalClassName != null) {
			Class loadedClass = loadSomeClass(principalClassName);
			Class signatures[] = { java.lang.String.class };
			Constructor constructorInstance = loadedClass
					.getConstructor(signatures);
			Object argumentsForConstructor[] = { username };
			principal = (Principal) constructorInstance
					.newInstance(argumentsForConstructor);
		} else {
			principal = new GeronimoUserPrincipal(username);
		}
		return principal;
	}

	private Class loadSomeClass(String classname) throws ClassNotFoundException {
		ClassLoader loader = Thread.currentThread().getContextClassLoader();
		return loader.loadClass(classname);
	}

	protected String getUsername() {
		return (getIdentity() != null) ? getIdentity().getName() : null;
	}

	protected Group createGroup(String name, Set principals) {
		Group roles = null;
		Iterator principalsIter = principals.iterator();
		while (principalsIter.hasNext()) {
			Object next = principalsIter.next();
			if (next instanceof Group) {
				Group nextGroup = (Group) next;
				if (nextGroup.getName().equals(name)) {
					roles = nextGroup;
					break;
				}
			}
		}
		if (roles == null) {
			roles = new GeronimoSimpleGroup(name);
			principals.add(roles);
		}
		return roles;
	}

	protected abstract String getUsersPassword() throws LoginException;

	protected abstract Group[] getRoleSets() throws LoginException;
}
