/**
 *  Licensed to the Apache Software Foundation (ASF) under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
 /*
 * Auto-generated by J2G Migration. 
 */
package org.apache.geronimo.j2g.sources.dependence.compatibility;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;

import org.apache.geronimo.util.encoders.Base64Encoder;

public class PasswordHasher {
	
	private final static String HEX_ENC = "HEX";
	private final static String ISO_ENC = "ISO-8859-1";
	private final static String BASE_64_ENC = "BASE64";
	
	public static String calculateHash(String algorithm, String encoding, String charset, String username, String password, DigestCallback callback) {
		String passwordHash = null;
		byte passBytes[];
		try {
			if (charset == null) {
				passBytes = password.getBytes();
			}
			else {
				passBytes = password.getBytes(charset);
			}
		} catch (UnsupportedEncodingException uee) {
			passBytes = password.getBytes();
		}
		try {
			MessageDigest mesDigest = MessageDigest.getInstance(algorithm);
			if (callback != null) {
				callback.preDigest(mesDigest);
			}
			mesDigest.update(passBytes);
			if (callback != null) {
				callback.postDigest(mesDigest);
			}
			byte hash[] = mesDigest.digest();
			if (encoding.equalsIgnoreCase(HEX_ENC)) {
				passwordHash = base16Encoding(hash);
			} else if (encoding.equalsIgnoreCase(BASE_64_ENC)) {
				passwordHash = base64Encoding(hash);
			}
		} catch (Exception e) {}
		return passwordHash;
	}

	private static String base16Encoding(byte encodedChain[]) {
		StringBuffer buffer = new StringBuffer(encodedChain.length * 2);
		for (int i = 0; i < encodedChain.length; i++) {
			byte nextByte = encodedChain[i];
			char nextChar = (char) (nextByte >> 4 & 0xf);
			if (nextChar > '\t') {
				nextChar = (char) ((nextChar - 10) + 97);
			}
			else {
				nextChar += '0';
			}
			buffer.append(nextChar);
			nextChar = (char) (nextByte & 0xf);
			if (nextChar > '\t') {
				nextChar = (char) ((nextChar - 10) + 97);
			}
			else {
				nextChar += '0';
			}
			buffer.append(nextChar);
		}
		return buffer.toString();
	}

	private static String base64Encoding(byte bytes[]) {
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		Base64Encoder encoder = new Base64Encoder();
		try {
			encoder.encode(bytes, 0, bytes.length, out);
		} catch (IOException e) {}
		String result = null;
		try {
			result = out.toString(ISO_ENC);
		} catch (UnsupportedEncodingException e) {}
		return result;
	}
}
